
importFrom("ellipse", ellipse)
importFrom("leaps", leaps)
importFrom("openxlsx", read.xlsx, addWorksheet, createStyle, createWorkbook, saveWorkbook, setColWidths, writeDataTable, writeData)
importFrom("robustbase", adjboxStats, mc)
importFrom("colorspace", sequential_hcl, diverge_hcl,
                         rainbow_hcl, terrain_hcl, heat_hcl)
importFrom("knitr", engine_output, "kable")
importFrom("kableExtra", kable_styling)
importFrom("MASS", kde2d)
importFrom(htmlwidgets, saveWidget, sizingPolicy)
importFrom(htmltools, attachDependencies, htmlDependency)
#importFrom("rlang", enquo, quo_is_missing, eval_tidy, as_label, caller_env, abort)
#importFrom("xts", endpoints, period.apply)
#importFrom("triangle", dtriangle, rtriangle)
#importFrom("sas7bdat", read.sas7bdat)
#importFrom("png", readPNG)
#importFrom("wesanderson", wes_palette)
#importFrom("rmarkdown", render, pandoc_available)
#importFrom("MBESS", ci.smd)
#importFrom("randomcoloR", distinctColorPalette)

importFrom("graphics", abline, arrows, axis, axis.Date, barplot, box, boxplot, grconvertX, grconvertY, hist, image, legend, lines, mtext, pairs, par, pie, plot, plot.default, plot.new, plot.window, points, polygon, rasterImage, rect, rug, segments, stripchart, strwidth, sunflowerplot, symbols, text, title, axTicks, smoothScatter, strheight, filled.contour, .filled.contour)

importFrom("grDevices", boxplot.stats, col2rgb, colorRampPalette, colors, dev.cur, dev.list, dev.new, dev.off, dev.set, gray, heat.colors, hcl, palette, pdf, rainbow, rgb, terrain.colors, as.graphicsAnnot, hcl.colors, palette.colors, gray.colors,
convertColor)

importFrom("stats", approx, binom.test, confint.default, IQR, TukeyHSD, addmargins, anova, aov, as.formula, bw.nrd0, chisq.test, coef, complete.cases, confint, contrasts, cooks.distance, cor, cor.test, cov, density, dffits, dlnorm, dnorm, dt, dunif, factanal, fitted, frequency, glm, heatmap, interaction.plot, lm, lm.influence, loadings, loess, lowess, median, mahalanobis, na.omit, pf, pnorm, prop.test, power.t.test, predict, printCoefmat, promax, pt, qnorm, qt, quantile, rbinom, replications, residuals, reformulate, rlnorm, rnorm, reshape, rstandard, rstudent, runif, sd, setNames, shapiro.test, start, stl, t.test, terms.formula, update, var, varimax, is.ts, time,  weighted.mean, fivenum, as.dist, hclust, cutree, aggregate, DF2formula, model.matrix, HoltWinters, ts, vcov, xtabs)

importFrom("utils", capture.output, head, help, packageVersion, read.csv, read.delim, read.fwf, read.table, type.convert, write.csv, write.table, browseURL, tail)

importFrom("methods", hasArg)

importFrom("lattice", xyplot, histogram, barchart, panel.grid, panel.xyplot, panel.loess, panel.lmline, panel.densityplot, panel.superpose, trellis.par.get, current.panel.limits, panel.text, panel.barchart, panel.histogram, panel.dotplot, panel.points, panel.polygon, panel.segments, panel.violin, panel.bwplot, panel.stripplot, panel.number, panel.lines)

importFrom("latticeExtra", panel.ellipse, panel.xyarea)

#importFrom("shiny", runApp)

S3method(print, out_all)
S3method(print, out)

# Export all names except those that begin with .
#exportPattern("^[^\\.]")
#
# Export all names
#exportPattern(".")


export (

  ANOVA, av, av_brief,
  BarChart,
  BoxPlot, bx,
  Chart,
  Correlation, cr, cr_brief,
  CountAll, ca,
  corCFA, cfa, scales,
  corPrint,
  corRead, rd.cor,
  corReorder, reord,
  corReflect, reflect, corProp, cp, corRead,
  corScree, scree, corEFA, efa,
  Density,
  details, db,
  factors,
  Flows,
  getColors,
  Histogram, hs,
  interact,
  kurtosis,
  label,
  Logit, lr,
  Merge, mrg,
  Model, model, model_brief,
  Nest,
  order_by,
  PieChart, pc,
  pivot,
  Plot,
  print.out_all, print.out,
  prob_norm, prob_znorm, prob_tcut,
  Prop_test, prop,
  Read, rd, rd_lbl, Read2,
  recode, rescale,
  Regression, reg, reg_brief, regPlot,
  rename,
  reshape_long, reshape_wide,
  savePlotly,
  showColors, showPalettes,
  simCImean, simCLT, simFlips, simMeans,
  ScatterPlot, sp,
  see, skew,
  Subset,
  SummaryStats, ss, ss_brief,
  STL,
  style,
  to,
  train_test,
  Transform,
  tsExtract,
  ttest, tt, tt_brief,
  ttestPower, ttp,
  values, VariableLabels, vl,
  ViolinPlot, vp,
  Write, wrt, wrt_r, wrt_x,
  X, XY,
  xAnd, xNum, xP, xRow, xU, xW,
  .
)
