scales <-
function(..., iter=0, resid=FALSE, item_cor=FALSE, sort=FALSE, heat_map=FALSE)

  corCFA(..., iter=0, resid=FALSE, item_cor=FALSE, sort=FALSE,
              heat_map=FALSE, fun_call=match.call())

