CountAll <-
function(x=d, quiet=FALSE, ...)  {

  manage.gr <- .graphman()  # see if graphics are to be managed
  if (manage.gr)
    cat("\n>>> Note: Must have pdf_file specified or run in RStudio.\n\n")


  # deparse/substitute must precede any local reassignment of x
  dname <- deparse(substitute(x))

  # let deprecated mydata work as default
  dfs <- .getdfs()
  if ("mydata" %in% dfs  &&  !("d" %in% dfs)) {
    x <- mydata
    dname <- "mydata"  # update after reassignment
  }

  options(dname = dname)

  if (missing(x)) {
    if (!exists(dname, where=.GlobalEnv)) {
      cat("\n"); stop(call.=FALSE, "\n","------\n",
        "Need to specify an existing data frame,\n",
        "or data frame  d  must exist.\n\n")
    }
  }
    
  if (!quiet) {
    cat("\n")
    .dash(25,"-")
    cat(format(Sys.time(), "%a %b %d, %Y at %H:%M"), "\n")
    .dash(25,"-")
  }

  if (!quiet) {
    cat("\n\n\n")
    .dash(37,"+")
    cat("Histogram for Each Numeric Variable\n")
    .dash(37,"+")
  }

  X(data=x, quiet=quiet, eval_df=FALSE, use_plotly=FALSE, ...)
  
  if (!quiet) {
    cat("\n\n\n")
    .dash(39,"+")
    cat("Bar Chart for Each Non-numeric Variable\n")
    .dash(39,"+")
  }
  Chart(data=x, quiet=quiet, eval_df=FALSE, use_plotly=FALSE,
           one_plot=FALSE, ...)
  
}
